from abc import ABC, abstractmethod
import sys


class LambdaFunction(ABC):
    def __init__(self):
        pass

    @classmethod
    def get_handler(cls, *args, **kwargs):
        def handler(event, context):
            return cls(*args, **kwargs).run(event, context)

        return handler

    @abstractmethod
    def run(self, event, context):
        raise NotImplementedError


def lambdahandler(func):
    # Creates dynamic handler in caller's module called MyCallingClassHandler
    module = func.__module__
    handler_name = f"{func.__name__}Handler"
    setattr(sys.modules[module], handler_name, func.get_handler())
    func.is_lambda = True
    return func


class QueueFunction(LambdaFunction):
    pass


class ScheduleFunction(LambdaFunction):
    pass


# def queuetrigger(queue):
#     """
#     Used to decorate a LambdaFunction to define an SQS trigger during deployment.
#     """
#     print("In decorator")
#     print(queue.__name__)

#     def decorator(func):
#         print("In nested decorator")
#         print(func)

#         def wrapper(*args, **kwargs):
#             print("In wrapper")
#             print(args)
#             print(kwargs)
#             return func(args[0])

#         return wrapper

#     print("Out of wrapper")
#     return decorator



from ...deployer import Deployer
import click


@click.command()
@click.option("--name", default="local", help="The name of the project to run.")
@click.option("--dir", default="./", help="The directory to deploy as a BLD project.")
@click.option(
    "--local", is_flag=True, default=False, help="Build locally instead of using SAM."
)
def start_api(name, dir, local):
    deployer = Deployer(name, dir, local=local)
    deployer.start_api()



from .function import LambdaFunction
import simplejson as json
import re
import decimal
from datetime import datetime


# class DecimalEncoder(json.JSONEncoder):
#     def _iterencode(self, o, markers=None):
#         if isinstance(o, decimal.Decimal):
#             return (str(o) for o in [o])
#         return super(DecimalEncoder, self)._iterencode(o, markers)


class APIFunction(LambdaFunction):
    def run(self, event, context):
        # Get the calling method and route appropriately.
        method = event["httpMethod"].lower()

        # Get get query params to pass in.
        params = self._get_url_params()
        args = [event["pathParameters"][p] for p in params]

        # TODO: Should use a more elegent method of routing here.
        if method == "post":
            response = self.post(event, context, *args)

        elif method == "get":
            response = self.get(event, context, *args)

        elif method == "put":
            response = self.put(event, context, *args)

        elif method == "delete":
            response = self.delete(event, context, *args)

        elif method == "options":
            response = self.create_return()

        else:
            raise Exception(f"Triggered with unexpected HTTP method {method}.")

        return response

    def create_return(self, status_code=200, body=None):
        return_obj = {"statusCode": status_code}
        headers = {
            "Access-Control-Allow-Origin": "*",
            "Access-Control-Allow-Credentials": "true",
            "Access-Control-Allow-Headers": "Authorization,Content-Type",
            "Access-Control-Allow-Methods": "*",
        }
        return_obj["headers"] = headers
        if body or isinstance(body, list):
            return_obj["body"] = json.dumps(body, default=datetime.isoformat)
        return return_obj

    def get_methods(self):
        methods = []
        # Check post method.
        post = getattr(self, "post", None)
        if callable(post):
            methods.append("POST")

        # Check get method.
        get = getattr(self, "get", None)
        if callable(get):
            methods.append("GET")

        # Check put method.
        put = getattr(self, "put", None)
        if callable(put):
            methods.append("PUT")

        # Check delete metho
        delete = getattr(self, "delete", None)
        if callable(delete):
            methods.append("DELETE")

        return methods

    def _get_url_params(self):
        if not self.endpoint.endswith("/"):
            self.endpoint = self.endpoint + "/"
        regex = re.compile("/{(.*?)}/")
        params = regex.findall(self.endpoint)
        return params



import click
from .deploy import commands as deploy
from .start_api import commands as start_api
from .invoke import commands as invoke


@click.group()
def entry_point():
    pass


entry_point.add_command(deploy.deploy)
entry_point.add_command(start_api.start_api)
entry_point.add_command(invoke.invoke)


if __name__ == "__main__":
    entry_point()
